@@ -20,11 +20,13 @@ grid, the time control panel, and the menu.
 """
 from functools import partial
 from ast import literal_eval
+from threading import Thread
 
 from kivy.app import App
 from kivy.clock import mainthread
 from kivy.factory import Factory
 from kivy.lang import Builder
+from kivy.uix.slider import Slider
 from kivy.uix.togglebutton import ToggleButton
 from kivy.uix.button import Button
 from kivy.uix.boxlayout import BoxLayout
@@ -68,9 +70,6 @@ class StatListPanel(BoxLayout):
 	engine = ObjectProperty()
 	proxy = ObjectProperty()
 	toggle_stat_cfg = ObjectProperty()
-	toggle_gridview = ObjectProperty()
-	toggle_calendar = ObjectProperty()
-	toggle_timestream = ObjectProperty()
 
 	def on_proxy(self, *args):
 		if hasattr(self.proxy, 'name'):
@@ -201,6 +200,7 @@ class MainScreen(Screen):
 	statlist = ObjectProperty()
 	statpanel = ObjectProperty()
 	timepanel = ObjectProperty()
+	turnscroll = ObjectProperty()
 	kv = StringProperty()
 	use_kv = BooleanProperty()
 	play_speed = NumericProperty()
@@ -315,8 +315,8 @@ class MainScreen(Screen):
 	def on_touch_down(self, touch):
 		if self.visible:
 			touch.grab(self)
-		for interceptor in (self.timepanel, self.charmenu, self.statpanel,
-							self.dummyplace, self.dummything):
+		for interceptor in (self.timepanel, self.turnscroll, self.charmenu,
+							self.statpanel, self.dummyplace, self.dummything):
 			if interceptor.collide_point(*touch.pos):
 				interceptor.dispatch('on_touch_down', touch)
 				self.boardview.keep_selection = self.gridview.keep_selection = True
@@ -328,6 +328,8 @@ class MainScreen(Screen):
 	def on_touch_up(self, touch):
 		if self.timepanel.collide_point(*touch.pos):
 			return self.timepanel.dispatch('on_touch_up', touch)
+		elif self.turnscroll.collide_point(*touch.pos):
+			return self.turnscroll.dispatch('on_touch_up', touch)
 		elif self.charmenu.collide_point(*touch.pos):
 			return self.charmenu.dispatch('on_touch_up', touch)
 		elif self.statpanel.collide_point(*touch.pos):
@@ -392,7 +394,7 @@ class MainScreen(Screen):
 		If you want to disable this, set ``engine.universal['block'] = True``
 
 		"""
-		if self.playbut.state == 'normal' or not hasattr(
+		if self.playbut is None or self.playbut.state == 'normal' or not hasattr(
 			self.app,
 			'engine') or self.app.engine is None or self.app.engine.closed:
 			return
@@ -433,7 +435,10 @@ class MainScreen(Screen):
 				"MainScreen: not advancing time while there's a dialog")
 			return
 		self.tmp_block = True
-		eng.next_turn(cb=partial(self._update_from_next_turn, cb=cb))
+		self._next_turn_thread = Thread(
+			target=eng.next_turn,
+			kwargs={'cb': partial(self._update_from_next_turn, cb=cb)})
+		self._next_turn_thread.start()
 		self.ids.charmenu._switch_to_menu()
 
 	def switch_to_calendar(self, *args):
@@ -475,6 +480,8 @@ class CharMenuContainer(BoxLayout):
 	dummyplace = ObjectProperty()
 	dummything = ObjectProperty()
 	portaladdbut = ObjectProperty()
+	toggle_gridview = ObjectProperty()
+	toggle_timestream = ObjectProperty()
 
 	def __init__(self, **kwargs):
 		super(CharMenuContainer, self).__init__(**kwargs)
@@ -486,10 +493,17 @@ class CharMenuContainer(BoxLayout):
 		self.charmenu.bind(dummything=self.setter('dummything'))
 		self.portaladdbut = self.charmenu.portaladdbut
 		self.charmenu.bind(portaladdbut=self.setter('portaladdbut'))
+		if self.toggle_gridview:
+			self.charmenu = self.toggle_gridview
+		self.bind(
+			toggle_gridview=self.charmenu.setter('toggle_gridview'),
+			toggle_timestream=self.charmenu.setter('toggle_timestream'))
 		self.stepper = RuleStepper(size_hint_y=0.9)
-		self.button = Button(on_release=self._toggle,
+		self.button = Button(on_release=self._toggle_stepper,
 								text='Rule\nstepper',
 								size_hint_y=0.1)
+		app = App.get_running_app()
+		app.bind(branch=self._switch_to_menu, turn=self._switch_to_menu)
 
 	def on_parent(self, *args):
 		if not self.screen or not hasattr(self, 'charmenu') or not hasattr(
@@ -500,7 +514,7 @@ class CharMenuContainer(BoxLayout):
 		self.add_widget(self.button)
 
 	@trigger
-	def _toggle(self, *args):
+	def _toggle_stepper(self, *args):
 		if self.charmenu in self.children:
 			engine = self.screen.app.engine
 			self.clear_widgets()
@@ -523,11 +537,57 @@ class CharMenuContainer(BoxLayout):
 			self.add_widget(self.button)
 
 
+class TurnScroll(Slider):
+
+	def __init__(self, **kwargs):
+		kwargs['step'] = 1
+		super().__init__(**kwargs)
+		self._collect_engine()
+
+	def _collect_engine(self, *args):
+		app = App.get_running_app()
+		if app.engine is None:
+			Clock.schedule_once(self._collect_engine, 0)
+			return
+		engine = app.engine
+		self.min = engine.initial_turn
+		self.max = engine.final_turn
+		Logger.debug(f"TurnScroll: {self.min}<-->{self.max}")
+		self.value = engine.turn
+		engine.time.connect(self._receive_time)
+
+	def _receive_time(self, engine, branch, turn, tick):
+		self.value = turn
+		try:
+			self.min = engine.initial_turn
+		except KeyError:
+			self.min = turn
+		try:
+			self.max = engine.final_turn
+		except KeyError:
+			self.max = turn
+		Logger.debug(f"TurnScroll: {self.min}<-{self.value}->{self.max}")
+
+	def on_touch_move(self, touch):
+		if touch.grab_current == self:
+			app = App.get_running_app()
+			app.mainscreen.timepanel.ids.turnfield.hint_text = str(
+				int(self.value))
+		return super().on_touch_move(touch)
+
+	def on_touch_up(self, touch):
+		if touch.grab_current == self:
+			app = App.get_running_app()
+			app.engine.time.disconnect(self._receive_time)
+			Logger.debug(f"TurnScroll: about to travel to {self.value}")
+			app.time_travel(app.engine.branch, int(self.value))
+			app.engine.time.connect(self._receive_time)
+
+
 Builder.load_string("""
 #: import resource_find kivy.resources.resource_find
 <StatListPanel>:
 	orientation: 'vertical'
-	cfgstatbut: cfgstatbut
 	statlist: statlist
 	id: statpanel
 	proxy: app.selected_proxy
@@ -537,25 +597,14 @@ Builder.load_string("""
 		bold: True
 	Calendar:
 		id: statlist
-		size_hint_y: 0.8
 		entity: root.proxy
 		update_mode: 'present'
 		disabled: app.edit_locked
-	Button:
-		id: gridviewbut
-		size_hint_y: 0.05
-		text: 'Toggle grid'
-		on_release: root.toggle_gridview()
 	Button:
 		id: cfgstatbut
 		size_hint_y: 0.05
 		text: root.button_text
 		on_release: root.toggle_stat_cfg()
-	Button:
-		id: timestreambut
-		size_hint_y: 0.05
-		text: 'Timestream'
-		on_release: root.toggle_timestream()
 <SimulateButton>:
 	graphics_top: self.y + self.font_size + (self.height - self.font_size) * (3/4)
 	graphics_bot: self.y + self.font_size + 3
@@ -653,11 +702,17 @@ Builder.load_string("""
 	statlist: statpanel.statlist
 	statpanel: statpanel
 	timepanel: timepanel
+	turnscroll: turnscroll
 	dialoglayout: dialoglayout
+	TurnScroll:
+		id: turnscroll
+		value_track: True
+		pos_hint: {'bot': 0}
+		size_hint: (1, 0.1)
 	Widget:
 		id: mainview
 		x: statpanel.right
-		y: 0
+		y: turnscroll.top
 		size_hint: (None, None)
 		width: charmenu.x - statpanel.right
 		height: root.height
@@ -665,24 +720,27 @@ Builder.load_string("""
 		id: statpanel
 		engine: app.engine
 		toggle_stat_cfg: app.statcfg.toggle
-		toggle_calendar: root.toggle_calendar
-		toggle_gridview: root.toggle_gridview
-		pos_hint: {'left': 0, 'top': 1}
-		size_hint: (0.25, 0.8)
+		x: 0
+		y: timepanel.top
+		size_hint: (0.25, 0.7)
 		selection_name: app.selected_proxy_name
 		toggle_timestream: root.toggle_timestream
 	TimePanel:
 		id: timepanel
 		screen: root
-		pos_hint: {'bot': 0}
+		x: 0
+		y: turnscroll.top
 		size_hint: (0.25, 0.2)
 		disable_one_turn: root.tmp_block
 	CharMenuContainer:
 		id: charmenu
+		toggle_calendar: root.toggle_calendar
+		toggle_gridview: root.toggle_gridview
+		toggle_timestream: root.toggle_timestream
 		orientation: 'vertical'
 		screen: root
 		pos_hint: {'right': 1, 'top': 1}
-		size_hint: (0.1, 1)
+		size_hint: (0.2, 0.9)
 	DialogLayout:
 		id: dialoglayout
 		engine: app.engine
